{% extends "wheelcms_axle/base.html" %}


{% block base_main %}

<div class="row-fluid">
  <div class="span12">
    <h3>{{content.title}}</h3>
  </div>
</div>
<div class="row-fluid">
  <div class="span-12">
    <dl>
    {% for c in content.items.all %}
      <dt><a href="{{c.node.get_absolute_url}}">{{c.title}}</a></dt>
      {# resolve a base content to a specific derived content through .content #}
      <dd>{{c.content.spoke.description|safe}}</dd>

    {% endfor %}
    </dl>
  </div>
</div>
{% endblock %}
